# Generated by Django 4.1.6 on 2023-03-17 14:44

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("netdoc", "0005_diagram_include_global_vrf_alter_diagram_details"),
    ]

    operations = [
        migrations.AlterField(
            model_name="routetableentry",
            name="metric",
            field=models.BigIntegerField(blank=True, editable=False, null=True),
        ),
    ]
