# Generated by Django 4.1.6 on 2023-03-23 11:15

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("netdoc", "0006_alter_routetableentry_metric"),
    ]

    operations = [
        migrations.AddField(
            model_name="discoverylog",
            name="supported",
            field=models.BooleanField(default=True, editable=False),
        ),
    ]
