{% extends 'base/layout.html' %}
{% load helpers %}
{% load render_table from django_tables2 %}
{% comment %}
__author__ = "Andrea Dainese"
__contact__ = "andrea@adainese.it"
__copyright__ = "Copyright 2022, Andrea Dainese"
__license__ = "GPLv3"
{% endcomment %}
{% comment %}
Replace "templates/generic/bulk_delete.html".
{% endcomment %}
{% block title %}Discover {{ table.rows|length }} {{ model|meta:"verbose_name_plural"|bettertitle }}?{% endblock %}
{% block content %}
    <div class="container-xl px-0">
        <div class="table-responsive">{% render_table table 'inc/table.html' %}</div>
        <div class="row mt-3">
            <form action="" method="post">
                {% csrf_token %}
                {% for field in form.hidden_fields %}{{ field }}{% endfor %}
                <div class="text-end">
                    <a href="{{ return_url }}" class="btn btn-outline-dark">Cancel</a>
                    <button type="submit" name="_confirm" class="btn btn-secondary">
                        Discover {{ table.rows|length }} {{ model|meta:"verbose_name_plural" }}
                    </button>
                </div>
            </form>
        </div>
    </div>
{% endblock content %}
