{% comment %}
__author__ = "Andrea Dainese"
__contact__ = "andrea@adainese.it"
__copyright__ = "Copyright 2022, Andrea Dainese"
__license__ = "GPLv3"

Used in templatetags/netdoc_buttons.py
{% endcomment %}
<a href="{{ url }}" class="btn btn-sm btn-purple">
    <span class="mdi mdi-download" aria-hidden="true"></span>&nbsp;Export
</a>
