# Generated by Django 4.1.6 on 2023-02-12 09:15

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("dcim", "0167_module_status"),
        ("ipam", "0063_standardize_description_comments"),
        ("netdoc", "0002_alter_arptableentry_custom_field_data_and_more"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="arptableentry",
            options={
                "ordering": ["ip_address"],
                "verbose_name": "ARP table entry",
                "verbose_name_plural": "ARP table entries",
            },
        ),
        migrations.AlterModelOptions(
            name="credential",
            options={
                "ordering": ["name"],
                "verbose_name": "Credential",
                "verbose_name_plural": "Credentials",
            },
        ),
        migrations.AlterModelOptions(
            name="discoverable",
            options={
                "ordering": ["mode", "address"],
                "verbose_name": "Device",
                "verbose_name_plural": "Devices",
            },
        ),
        migrations.AlterModelOptions(
            name="discoverylog",
            options={
                "ordering": ["created"],
                "verbose_name": "Log",
                "verbose_name_plural": "Logs",
            },
        ),
        migrations.AlterModelOptions(
            name="routetableentry",
            options={
                "ordering": ("device", "protocol", "metric"),
                "verbose_name": "Route ",
                "verbose_name_plural": "Routes",
            },
        ),
        migrations.RenameField(
            model_name="routetableentry",
            old_name="type",
            new_name="protocol",
        ),
        migrations.AlterUniqueTogether(
            name="routetableentry",
            unique_together=set(),
        ),
        migrations.RemoveField(
            model_name="discoverylog",
            name="request",
        ),
        migrations.AddField(
            model_name="discoverylog",
            name="details",
            field=models.JSONField(default=dict, editable=False),
        ),
        migrations.AddField(
            model_name="discoverylog",
            name="order",
            field=models.IntegerField(default=128),
        ),
        migrations.AddField(
            model_name="discoverylog",
            name="template",
            field=models.CharField(
                default="migrations", editable=False, max_length=255
            ),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name="discoverylog",
            name="raw_output",
            field=models.JSONField(default=dict),
        ),
        migrations.AlterField(
            model_name="routetableentry",
            name="distance",
            field=models.IntegerField(blank=True, editable=False, null=True),
        ),
        migrations.AlterField(
            model_name="routetableentry",
            name="metric",
            field=models.IntegerField(blank=True, editable=False, null=True),
        ),
        migrations.AlterUniqueTogether(
            name="routetableentry",
            unique_together={
                ("device", "destination", "distance", "metric", "protocol", "vrf")
            },
        ),
    ]
