# Generated by Django 4.1.6 on 2023-02-21 07:21

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("netdoc", "0004_alter_macaddresstableentry_options_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="diagram",
            name="include_global_vrf",
            field=models.BooleanField(default=False),
        ),
        migrations.AlterField(
            model_name="diagram",
            name="details",
            field=models.JSONField(default=dict),
        ),
    ]
